#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Implementations of CourseInfo and associated objects.
"""

from persistent import Persistent

from zope.container.btree import BTreeContainer
from zope.container.contained import Contained, NameChooser
from zope.interface import implements
from zope.lifecycleevent.interfaces import IObjectAddedEvent

from schooltool.app.app import InitBase, StartUpBase
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.courseinfo import CourseinfoMessage as _

import interfaces


COURSEINFO_ROOT_KEY = 'schooltool.courseinfo'


class CourseInfoRoot(BTreeContainer):
    """Root container of the schooltool.courseinfo."""

    implements(interfaces.ICourseInfoRoot)

class CourseInfoSchoolYearContainer(BTreeContainer):
    """Container of CourseInfoSchoolYear objects."""

    implements(interfaces.ICourseInfoSchoolYearContainer)

class CourseInfoSchoolYear(BTreeContainer):
    """Container of CourseInfo containers."""

    implements(interfaces.ICourseInfoSchoolYear)


class CourseInfoContainer(BTreeContainer):
    """Container of CourseInfo objects."""

    implements(interfaces.ICourseInfoContainer)

class CurrentCourseInfoContainer(BTreeContainer):
    """Container of the current year's CourseInfo objects."""

    implements(interfaces.ICurrentCourseInfoContainer)

class ProposedCourseInfoContainer(BTreeContainer):
    """Container of the next year's (proposed) CourseInfo objects."""

    implements(interfaces.IProposedCourseInfoContainer)

class CourseInfo(Persistent, Contained):
    """The main course info object."""

    implements(interfaces.ICourseInfo)

    def __init__(self, course_id, title, description, start_grade, end_grade,
                 duration, credits, elective, division, cluster='', pathway='',
                 requirements=[], government_id='', credit_codes=[],
                 supervisor='', planning_information=[], weight=1.0,
                 lab_fee=0.0, notes='', prerequisite='', outcomes=''):
        self.course_id = course_id
        self.title = title
        self.description = description
        self.start_grade = start_grade
        self.end_grade = end_grade
        self.duration = duration
        self.credits = credits
        self.elective = elective
        self.division = division
        self.cluster = cluster
        self.pathway = pathway
        self.requirements = requirements
        self.government_id = government_id
        self.credit_codes = credit_codes
        self.supervisor = supervisor
        self.planning_information = planning_information
        self.weight = weight
        self.lab_fee = lab_fee
        self.notes = notes
        self.prerequisite = prerequisite
        self.outcomes = outcomes

class CurrentCourseInfo(CourseInfo):
    """The course info object for a course in the current year. """

    implements(interfaces.ICurrentCourseInfo)

    def __init__(self, course_id, title, description, start_grade, end_grade,
                 duration, credits, elective, division, cluster='', pathway='',
                 requirements=[], government_id='', credit_codes=[],
                 supervisor='', planning_information=[], weight=1.0,
                 lab_fee=0.0, notes='', prerequisite='', outcomes=''):
        super(CurrentCourseInfo, self).__init__(
            course_id, title, description, start_grade, end_grade,
            duration, credits, elective, division, cluster=cluster, pathway=pathway,
            requirements=requirements, government_id=government_id, credit_codes=credit_codes,
            supervisor=supervisor, planning_information=planning_information, weight=weight,
            lab_fee=lab_fee, notes=notes, prerequisite=prerequisite, outcomes=outcomes)


class ProposedCourseInfo(CourseInfo):
    """The course info object for a course in the next year. """

    implements(interfaces.IProposedCourseInfo)

    def __init__(self, course_id, title, description, start_grade, end_grade,
                 duration, credits, elective, division, cluster='', pathway='',
                 requirements=[], government_id='', credit_codes=[],
                 supervisor='', planning_information=[], weight=1.0,
                 lab_fee=0.0, notes='', prerequisite='', outcomes=''):
        super(ProposedCourseInfo, self).__init__(
            course_id, title, description, start_grade, end_grade,
            duration, credits, elective, division, cluster=cluster, pathway=pathway,
            requirements=requirements, government_id=government_id, credit_codes=credit_codes,
            supervisor=supervisor, planning_information=planning_information, weight=weight,
            lab_fee=lab_fee, notes=notes, prerequisite=prerequisite, outcomes=outcomes)

def setUpCourseInfo(app):
    app[COURSEINFO_ROOT_KEY] = CourseInfoRoot()


class CourseInfoInit(InitBase):
    """Create the CourseInfoRoot object."""

    def __call__(self):
        setUpCourseInfo(self.app)


class CourseInfoStartup(StartUpBase):
    def __call__(self):
        if COURSEINFO_ROOT_KEY not in self.app:
            setUpCourseInfo(self.app)

